@@ -1063,30 +1063,7 @@ class TestPowerVMDriver(test.TestCase):
                          block_device_info=mock_bdms)
         assert_not_called()
 
-        mock_resp = mock.Mock()
-        mock_resp.status = 404
-        mock_resp.reqpath = (
-            '/rest/api/uom/ManagedSystem/c5d782c7-44e4-3086-ad15-'
-            'b16fb039d63b/LogicalPartition/1B5FB633-16D1-4E10-A14'
-            '5-E6FB905161A3?group=None')
-        mock_pvmuuid.side_effect = pvm_exc.HttpError(mock_resp)
-
-        # Invoke the method.
-        self.drv.destroy('context', self.inst, [],
-                         block_device_info=mock_bdms)
-        assert_not_called()
-
-        # Ensure the exception is raised with non-matching path
         reset_mocks()
-        mock_resp.reqpath = (
-            '/rest/api/uom/ManagedSystem/c5d782c7-44e4-3086-ad15-'
-            'b16fb039d63b/SomeResource/1B5FB633-16D1-4E10-A14'
-            '5-E6FB905161A3?group=None')
-        # Invoke the method.
-        self.assertRaises(exc.InstanceTerminationFailure,
-                          self.drv.destroy, 'context', self.inst,
-                          [], block_device_info=mock_bdms)
-        assert_not_called()
 
         # Test generic exception
         mock_pvmuuid.side_effect = ValueError('Some error')
@@ -1891,7 +1868,7 @@ class TestPowerVMDriver(test.TestCase):
             'block_mig', 'migrate_data')
         self.lpm.migration_abort.assert_called_once_with()
         mock_rec_meth.assert_called_once_with(
-            'context', self.lpm_inst, 'dest', mock.ANY, mock.ANY)
+            'context', self.lpm_inst, 'dest', migrate_data=mock.ANY)
         self.lpm.rollback_live_migration.assert_called_once_with('context')
         self.assertEqual(0, mock_post_meth.call_count)
 
@@ -1905,7 +1882,7 @@ class TestPowerVMDriver(test.TestCase):
             'context', self.lpm_inst, 'dest', mock_post_meth, mock_rec_meth,
             'block_mig', 'migrate_data')
         mock_rec_meth.assert_called_once_with(
-            'context', self.lpm_inst, 'dest', mock.ANY, mock.ANY)
+            'context', self.lpm_inst, 'dest', migrate_data=mock.ANY)
         self.lpm.rollback_live_migration.assert_called_once_with('context')
         self.assertEqual(0, mock_post_meth.call_count)
 
@@ -1921,7 +1898,7 @@ class TestPowerVMDriver(test.TestCase):
             'context', self.lpm_inst, 'dest', mock_post_meth, mock_rec_meth,
             'block_mig', 'migrate_data')
         mock_rec_meth.assert_called_once_with(
-            'context', self.lpm_inst, 'dest', mock.ANY, mock.ANY)
+            'context', self.lpm_inst, 'dest', migrate_data=mock.ANY)
         self.lpm.rollback_live_migration.assert_called_once_with('context')
         self.assertEqual(0, mock_post_meth.call_count)
 
@@ -2074,3 +2051,7 @@ class TestPowerVMDriver(test.TestCase):
         mock_bk_dev.return_value = 'info'
         self.assertEqual('info',
                          self.drv._get_block_device_info('ctx', self.inst))
+
+    def test_deallocate_networks_on_reschedule(self):
+        candeallocate = self.drv.deallocate_networks_on_reschedule(mock.Mock())
+        self.assertTrue(candeallocate)
