@@ -19,6 +19,7 @@ import mock
 
 import logging
 from nova import test
+from oslo_serialization import jsonutils
 import pypowervm.tests.test_fixtures as pvm_fx
 
 from nova_powervm.virt.powervm import host as pvm_host
@@ -30,13 +31,20 @@ logging.basicConfig()
 class TestPowerVMHost(test.TestCase):
     def test_host_resources(self):
         # Create objects to test with
+        sriov_adaps = [
+            mock.Mock(sriov_adap_id=1, phys_ports=[
+                mock.Mock(port_id=2, label='foo', supp_max_lps=1),
+                mock.Mock(port_id=3, label='', supp_max_lps=2)]),
+            mock.Mock(sriov_adap_id=4, phys_ports=[
+                mock.Mock(port_id=5, label='bar', supp_max_lps=3)])]
         ms_wrapper = mock.MagicMock(
             proc_units_configurable=500,
             proc_units_avail=500,
             memory_configurable=5242880,
             memory_free=5242752,
             mtms=mock.MagicMock(mtms_str='8484923A123456'),
-            memory_region_size='big')
+            memory_region_size='big',
+            asio_config=mock.Mock(sriov_adapters=sriov_adaps))
 
         # Run the actual test
         stats = pvm_host.build_host_resource_from_ms(ms_wrapper)
@@ -47,7 +55,7 @@ class TestPowerVMHost(test.TestCase):
                   ('memory_mb', 5242880), ('memory_mb_used', 128),
                   'hypervisor_type', 'hypervisor_version',
                   'hypervisor_hostname', 'cpu_info',
-                  'supported_instances', 'stats')
+                  'supported_instances', 'stats', 'pci_passthrough_devices')
         for fld in fields:
             if isinstance(fld, tuple):
                 value = stats.get(fld[0], None)
@@ -64,6 +72,22 @@ class TestPowerVMHost(test.TestCase):
             else:
                 value = stats['stats'].get(stat, None)
                 self.assertIsNotNone(value)
+        # pci_passthrough_devices.  Parse json - entries can be in any order.
+        ppdstr = stats['pci_passthrough_devices']
+        ppdlist = jsonutils.loads(ppdstr)
+        self.assertEqual({'foo', 'bar', 'default'}, {ppd['physical_network']
+                                                     for ppd in ppdlist})
+        self.assertEqual({'foo', 'bar', 'default'}, {ppd['label']
+                                                     for ppd in ppdlist})
+        self.assertEqual({'*:1:2.0', '*:1:3.0', '*:1:3.1', '*:4:5.0',
+                          '*:4:5.1', '*:4:5.2'},
+                         {ppd['address'] for ppd in ppdlist})
+        for ppd in ppdlist:
+            self.assertEqual('type-VF', ppd['dev_type'])
+            self.assertEqual('*:*:*.*', ppd['parent_addr'])
+            self.assertEqual('*', ppd['vendor_id'])
+            self.assertEqual('*', ppd['product_id'])
+            self.assertEqual(1, ppd['numa_node'])
 
 
 class TestHostCPUStats(test.TestCase):
@@ -241,7 +265,7 @@ class TestHostCPUStats(test.TestCase):
         prev_lpar_sample = mock_prev_phyp.sample.lpars[0].processor
         prev_lpar_sample.util_cap_proc_cycles = 0
         prev_lpar_sample.util_uncap_proc_cycles = 0
-        prev_lpar_sample.donated_proc_cycles = 0
+        prev_lpar_sample.idle_proc_cycles = 0
         delta3 = host_stats._delta_proc_cycles(mock_phyp.sample.lpars,
                                                mock_prev_phyp.sample.lpars)
         self.assertEqual(0, delta3)
@@ -254,11 +278,11 @@ class TestHostCPUStats(test.TestCase):
         mock_phyp, mock_prev_phyp = self._get_mock_phyps()
         mock_phyp.sample.lpars[0].processor.util_cap_proc_cycles = 250000
         mock_phyp.sample.lpars[0].processor.util_uncap_proc_cycles = 250000
-        mock_phyp.sample.lpars[0].processor.donated_proc_cycles = 500
+        mock_phyp.sample.lpars[0].processor.idle_proc_cycles = 500
         mock_prev_phyp.sample.lpars[0].processor.util_cap_proc_cycles = 0
         num = 455000
         mock_prev_phyp.sample.lpars[0].processor.util_uncap_proc_cycles = num
-        mock_prev_phyp.sample.lpars[0].processor.donated_proc_cycles = 1000
+        mock_prev_phyp.sample.lpars[0].processor.idle_proc_cycles = 1000
 
         # Test that a previous sample allows us to gather just the delta.
         new_elem = self._get_sample(4, mock_phyp.sample)
@@ -371,13 +395,13 @@ class TestHostCPUStats(test.TestCase):
         mock_lpar_4A.processor = mock.MagicMock(
             util_cap_proc_cycles=5005045000,
             util_uncap_proc_cycles=5005045000,
-            donated_proc_cycles=10000)
+            idle_proc_cycles=10000)
         mock_lpar_4A_prev = mock.Mock()
         mock_lpar_4A_prev.configure_mock(id=4, name='A')
         mock_lpar_4A_prev.processor = mock.MagicMock(
             util_cap_proc_cycles=40000,
             util_uncap_proc_cycles=40000,
-            donated_proc_cycles=0)
+            idle_proc_cycles=0)
         mock_phyp = mock.MagicMock(sample=mock.MagicMock(lpars=[mock_lpar_4A]))
         mock_prev_phyp = mock.MagicMock(
             sample=mock.MagicMock(lpars=[mock_lpar_4A_prev]))
