@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -18,6 +18,7 @@ import mock
 
 from nova import exception
 from nova.network import model
+from nova.network.neutronv2 import api as netapi
 from nova import test
 from oslo_config import cfg
 from pypowervm import exceptions as pvm_ex
@@ -39,6 +40,16 @@ def cna(mac):
     return nic
 
 
+class FakeNetworkAPI(object):
+    def __init__(self, physnet):
+        self.physical_network = physnet
+
+    def get(self, context, netid):
+        physnet = mock.MagicMock()
+        physnet.physical_network = self.physical_network
+        return physnet
+
+
 class TestVifFunctions(test.TestCase):
 
     def setUp(self):
@@ -52,7 +63,7 @@ class TestVifFunctions(test.TestCase):
     @mock.patch('pypowervm.wrappers.event.Event')
     def test_push_vif_event(self, mock_event, mock_dumps):
         mock_vif = mock.Mock(mac='MAC', href='HREF')
-        vif._push_vif_event(self.adpt, 'action', mock_vif,
+        vif._push_vif_event(self.adpt, 'action', mock_vif, mock.Mock(),
                             'pvm_sea')
         mock_dumps.assert_called_once_with(
             {'provider': 'NOVA_PVM_VIF', 'action': 'action', 'mac': 'MAC',
@@ -68,7 +79,7 @@ class TestVifFunctions(test.TestCase):
         # Exception reraises
         mock_event.bld.return_value.create.side_effect = IndexError
         self.assertRaises(IndexError, vif._push_vif_event, self.adpt, 'action',
-                          mock_vif, 'pvm_sea')
+                          mock_vif, mock.Mock(), 'pvm_sea')
         mock_dumps.assert_called_once_with(
             {'provider': 'NOVA_PVM_VIF', 'action': 'action', 'mac': 'MAC',
              'type': 'pvm_sea'})
@@ -94,7 +105,7 @@ class TestVifFunctions(test.TestCase):
                                                                new_vif=True)
         slot_mgr.register_vnet.assert_called_once_with(
             mock_bld_drv.return_value.plug.return_value)
-        mock_event.assert_called_once_with(self.adpt, 'plug', vnet,
+        mock_event.assert_called_once_with(self.adpt, 'plug', vnet, mock.ANY,
                                            'pvm_sea')
         self.assertEqual(mock_bld_drv.return_value.plug.return_value, vnet)
 
@@ -137,7 +148,7 @@ class TestVifFunctions(test.TestCase):
             mock_vif, cna_w_list=None)
         slot_mgr.drop_vnet.assert_called_once_with('vnet_w')
         mock_event.assert_called_once_with(self.adpt, 'unplug', 'vnet_w',
-                                           'pvm_sea')
+                                           mock.ANY, 'pvm_sea')
 
         # Clean up
         mock_bld_drv.reset_mock()
@@ -202,9 +213,10 @@ class TestVifFunctions(test.TestCase):
         mock_crt.return_value = mock.Mock()
         self.slot_mgr.build_map.get_mgmt_vea_slot = mock.Mock(
             return_value=(None, None))
+        mock_instance = mock.MagicMock(system_metadata={})
 
         # Run the method
-        vif.plug_secure_rmc_vif(self.adpt, 'instance', 'host_uuid',
+        vif.plug_secure_rmc_vif(self.adpt, mock_instance, 'host_uuid',
                                 self.slot_mgr)
 
         # Validate responses
@@ -222,9 +234,10 @@ class TestVifFunctions(test.TestCase):
         mock_crt.return_value = mock.Mock()
         self.slot_mgr.build_map.get_mgmt_vea_slot = mock.Mock(
             return_value=('mac_addr', 5))
+        mock_instance = mock.MagicMock(system_metadata={})
 
         # Run the method
-        vif.plug_secure_rmc_vif(self.adpt, 'instance', 'host_uuid',
+        vif.plug_secure_rmc_vif(self.adpt, mock_instance, 'host_uuid',
                                 self.slot_mgr)
 
         # Validate responses
@@ -233,6 +246,37 @@ class TestVifFunctions(test.TestCase):
             crt_vswitch=True, slot_num=5, mac_addr='mac_addr')
         self.assertFalse(self.slot_mgr.called)
 
+    @mock.patch('pypowervm.tasks.cna.crt_cna')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    def test_plug_secure_rmc_vif_for_rebuild(self, mock_pvm_uuid, mock_crt):
+        # Mock up the data
+        mock_pvm_uuid.return_value = 'lpar_uuid'
+        mock_crt.return_value = mock.Mock()
+        self.slot_mgr.build_map.get_mgmt_vea_slot = mock.Mock(
+            return_value=(None, None))
+        mock_instance = mock.MagicMock(
+            system_metadata={'mgmt_interface_mac': 'old_mac'})
+
+        # Run the method
+        vif.plug_secure_rmc_vif(self.adpt, mock_instance, 'host_uuid',
+                                self.slot_mgr)
+
+        # Validate responses
+        # Validate that as part of rebuild, pvm_cna.crt_cna is called with
+        # 'old_mac' stored in instance's syetem_metadata. Also, the slot
+        # number is not passed. This is because as part of rebuild, the
+        # instance is destroyed and spawned again. When the instance is
+        # destroyed, the slot data is removed. When instance is spawned,
+        # the required volume and network info is got as part of BDM
+        # and network info dicts. The only missing information is mgmt
+        # interface mac address.
+        mock_crt.assert_called_once_with(
+            self.adpt, 'host_uuid', 'lpar_uuid', 4094, vswitch='MGMTSWITCH',
+            crt_vswitch=True, slot_num=None, mac_addr='old_mac')
+        # Validate that register_cna is called.
+        self.slot_mgr.register_cna.assert_called_once_with(
+            mock_crt.return_value)
+
     def test_build_vif_driver(self):
         # Test the Shared Ethernet Adapter type VIF
         mock_inst = mock.MagicMock()
@@ -312,19 +356,115 @@ class TestVifSriovDriver(test.TestCase):
         self.inst = mock.MagicMock()
         self.drv = vif.PvmVnicSriovVifDriver(self.adpt, 'host_uuid', self.inst)
 
-    def test_plug_no_pports(self):
+    @mock.patch('pypowervm.wrappers.managed_system.System.get')
+    def test_plug_no_pports(self, mock_sysget):
         """Raise when plug is called with a network with no physical ports."""
+        sriov_adaps = [
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='loc1', label='foo'),
+                mock.Mock(loc_code='loc2', label='')]),
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='loc3', label='bar'),
+                mock.Mock(loc_code='loc4', label='foo')])]
+        sys = mock.Mock(asio_config=mock.Mock(sriov_adapters=sriov_adaps))
+        mock_sysget.return_value = [sys]
         self.assertRaises(exception.VirtualInterfacePlugException,
                           self.drv.plug, FakeDirectVif('net2', pports=[]), 1)
 
+    @mock.patch('pypowervm.wrappers.iocard.VNIC.bld')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.tasks.sriov.set_vnic_back_devs')
+    @mock.patch('pypowervm.wrappers.managed_system.System.get')
+    def test_plug_no_physnet(self, mock_sysget, mock_back_devs, mock_pvm_uuid,
+                             mock_vnic_bld):
+        slot = 10
+        pports = ['port1', 'port2']
+        sriov_adaps = [
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port11', label='default'),
+                mock.Mock(loc_code='port3', label='data1')]),
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port4', label='data2'),
+                mock.Mock(loc_code='port22', label='default')])]
+        sys = mock.Mock(asio_config=mock.Mock(sriov_adapters=sriov_adaps))
+        mock_sysget.return_value = [sys]
+        netapi.API = mock.Mock(return_value=FakeNetworkAPI('default'))
+        self.drv.plug(FakeDirectVif('', pports=pports), slot)
+        # Ensure back devs are created with pports from sriov_adaps and
+        # not with what pports passed into plug method
+        mock_back_devs.assert_called_once_with(
+            mock_vnic_bld.return_value, ['port11', 'port22'], redundancy=3,
+            capacity=None, check_port_status=True,
+            sys_w=sys)
+
+    @mock.patch('pypowervm.wrappers.iocard.VNIC.bld')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.tasks.sriov.set_vnic_back_devs')
+    @mock.patch('pypowervm.wrappers.managed_system.System.get')
+    def test_plug_no_matching_pports(self, mock_sysget, mock_back_devs,
+                                     mock_pvm_uuid, mock_vnic_bld):
+        slot = 10
+        pports = ['port1', 'port2']
+        sriov_adaps = [
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port1', label='data1'),
+                mock.Mock(loc_code='port3', label='data1')]),
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port4', label='data2'),
+                mock.Mock(loc_code='port2', label='data2')])]
+        sys = mock.Mock(asio_config=mock.Mock(sriov_adapters=sriov_adaps))
+        mock_sysget.return_value = [sys]
+        netapi.API = mock.Mock(return_value=FakeNetworkAPI('default'))
+        # Ensure Plug exception is raised when there are no matching pports
+        # for physical network of corresponding neutron network
+        self.assertRaises(exception.VirtualInterfacePlugException,
+                          self.drv.plug,
+                          FakeDirectVif('default', pports=pports), slot)
+
+    @mock.patch('pypowervm.wrappers.iocard.VNIC.bld')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.tasks.sriov.set_vnic_back_devs')
+    @mock.patch('pypowervm.wrappers.managed_system.System.get')
+    def test_plug_bad_pports(self, mock_sysget, mock_back_devs, mock_pvm_uuid,
+                             mock_vnic_bld):
+        slot = 10
+        pports = ['bad1', 'bad2']
+        sriov_adaps = [
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port1', label='default'),
+                mock.Mock(loc_code='port3', label='data1')]),
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port4', label='data2'),
+                mock.Mock(loc_code='port2', label='default')])]
+        sys = mock.Mock(asio_config=mock.Mock(sriov_adapters=sriov_adaps))
+        mock_sysget.return_value = [sys]
+        netapi.API = mock.Mock(return_value=FakeNetworkAPI('default'))
+        self.drv.plug(FakeDirectVif('', pports=pports), slot)
+        # Ensure back devs are created with correct pports belonging to same
+        # physical network corresonding to neutron network
+        mock_back_devs.assert_called_once_with(
+            mock_vnic_bld.return_value, ['port1', 'port2'], redundancy=3,
+            capacity=None, check_port_status=True,
+            sys_w=sys)
+
+    @mock.patch('pypowervm.wrappers.managed_system.System.get')
     @mock.patch('pypowervm.util.sanitize_mac_for_api')
     @mock.patch('pypowervm.wrappers.iocard.VNIC.bld')
     @mock.patch('pypowervm.tasks.sriov.set_vnic_back_devs')
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
     def test_plug(self, mock_pvm_uuid, mock_back_devs, mock_vnic_bld,
-                  mock_san_mac):
+                  mock_san_mac, mock_sysget):
         slot = 10
         pports = ['port1', 'port2']
+        sriov_adaps = [
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port1', label='default'),
+                mock.Mock(loc_code='port3', label='data1')]),
+            mock.Mock(phys_ports=[
+                mock.Mock(loc_code='port4', label='data2'),
+                mock.Mock(loc_code='port2', label='default')])]
+        sys = mock.Mock(asio_config=mock.Mock(sriov_adapters=sriov_adaps))
+        mock_sysget.return_value = [sys]
         self.drv.plug(FakeDirectVif('default', pports=pports),
                       slot)
         mock_san_mac.assert_called_once_with('ab:ab:ab:ab:ab:ab')
@@ -334,7 +474,8 @@ class TestVifSriovDriver(test.TestCase):
             allowed_macs='NONE')
         mock_back_devs.assert_called_once_with(
             mock_vnic_bld.return_value, ['port1', 'port2'], redundancy=3,
-            capacity=None, check_port_status=True)
+            capacity=None, check_port_status=True,
+            sys_w=sys)
         mock_pvm_uuid.assert_called_once_with(self.drv.instance)
         mock_vnic_bld.return_value.create.assert_called_once_with(
             parent_type=pvm_lpar.LPAR, parent_uuid=mock_pvm_uuid.return_value)
@@ -353,7 +494,8 @@ class TestVifSriovDriver(test.TestCase):
             allowed_macs='NONE')
         mock_back_devs.assert_called_once_with(
             mock_vnic_bld.return_value, ['port1', 'port2'],
-            redundancy=3, capacity=0.08, check_port_status=True)
+            redundancy=3, capacity=0.08, check_port_status=True,
+            sys_w=sys)
         mock_pvm_uuid.assert_called_once_with(self.drv.instance)
         mock_vnic_bld.return_value.create.assert_called_once_with(
             parent_type=pvm_lpar.LPAR, parent_uuid=mock_pvm_uuid.return_value)
