@@ -43,6 +43,31 @@ class DiskType(object):
     IMAGE = 'image'
 
 
+class IterableToFileAdapter(object):
+    """A degenerate file-like so that an iterable can be read like a file.
+
+    The Glance client returns an iterable, but PowerVM requires a file.  This
+    is the adapter between the two.
+
+    Taken from xenapi/image/apis.py
+    """
+
+    def __init__(self, iterable):
+        self.iterator = iterable.__iter__()
+        self.remaining_data = ''
+
+    def read(self, size):
+        chunk = self.remaining_data
+        try:
+            while not chunk:
+                chunk = next(self.iterator)
+        except StopIteration:
+            return ''
+        return_value = chunk[0:size]
+        self.remaining_data = chunk[size:]
+        return return_value
+
+
 @six.add_metaclass(abc.ABCMeta)
 class DiskAdapter(object):
 
@@ -157,9 +182,8 @@ class DiskAdapter(object):
             lpar_wrap = vm.get_instance_wrapper(self.adapter, instance)
         match_func = self.disk_match_func(disk_type, instance)
         for vios_uuid in self.vios_uuids:
-            vios_wrap = pvm_vios.VIOS.wrap(self.adapter.read(
-                pvm_vios.VIOS.schema_type, root_id=vios_uuid,
-                xag=[pvm_const.XAG.VIO_SMAP]))
+            vios_wrap = pvm_vios.VIOS.get(
+                self.adapter, uuid=vios_uuid, xag=[pvm_const.XAG.VIO_SMAP])
             for scsi_map in tsk_map.find_maps(
                     vios_wrap.scsi_mappings, client_lpar_id=lpar_wrap.id,
                     match_func=match_func):
