@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -16,7 +16,10 @@
 #
 
 import abc
+import six
+
 from nova import exception
+from nova.objects import migrate_data as mig_obj
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from pypowervm.tasks import management_console as mgmt_task
@@ -24,10 +27,8 @@ from pypowervm.tasks import migration as mig
 from pypowervm.tasks import storage as stor_task
 from pypowervm.tasks import vterm
 from pypowervm import util
-import six
 
 from nova_powervm import conf as cfg
-from nova_powervm.objects import migrate_data as mig_obj
 from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
@@ -124,9 +125,9 @@ class LiveMigrationDest(LiveMigration):
         self.mig_data.dest_proc_compat = (
             ','.join(self.drvr.host_wrapper.proc_compat_modes))
 
-        LOG.debug('src_compute_info: %s' % src_compute_info)
-        LOG.debug('dst_compute_info: %s' % dst_compute_info)
-        LOG.debug('Migration data: %s' % self.mig_data)
+        LOG.debug('src_compute_info: %s', src_compute_info)
+        LOG.debug('dst_compute_info: %s', dst_compute_info)
+        LOG.debug('Migration data: %s', self.mig_data)
 
         return self.mig_data
 
@@ -144,7 +145,7 @@ class LiveMigrationDest(LiveMigration):
         """
         LOG.debug('Running pre live migration on destination.',
                   instance=self.instance)
-        LOG.debug('Migration data: %s' % migrate_data)
+        LOG.debug('Migration data: %s', migrate_data, instance=self.instance)
 
         # Set the ssh auth key.
         mgmt_task.add_authorized_key(self.drvr.adapter,
@@ -162,12 +163,12 @@ class LiveMigrationDest(LiveMigration):
         # For each volume, make sure it's ready to migrate
         for vol_drv in vol_drvs:
             LOG.info(_LI('Performing pre migration for volume %(volume)s'),
-                     dict(volume=vol_drv.volume_id))
+                     dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
                 vol_drv.pre_live_migration_on_destination(
                     migrate_data.vol_data)
             except Exception as e:
-                LOG.exception(e)
+                LOG.exception(e, instance=self.instance)
                 # It failed.
                 vol_exc = LiveMigrationVolume(
                     host=self.drvr.host_wrapper.system_name,
@@ -199,11 +200,11 @@ class LiveMigrationDest(LiveMigration):
         # For each volume, make sure it completes the migration
         for vol_drv in vol_drvs:
             LOG.info(_LI('Performing post migration for volume %(volume)s'),
-                     dict(volume=vol_drv.volume_id))
+                     dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
                 vol_drv.post_live_migration_at_destination(mig_vol_stor)
             except Exception as e:
-                LOG.exception(e)
+                LOG.exception(e, instance=self.instance)
                 # It failed.
                 raise LiveMigrationVolume(
                     host=self.drvr.host_wrapper.system_name,
@@ -235,13 +236,15 @@ class LiveMigrationDest(LiveMigration):
         :param vol_drv: volume driver for the attached volume
         """
         LOG.info(_LI('Performing detach for volume %(volume)s'),
-                 dict(volume=vol_drv.volume_id))
-        try:
-            vol_drv.cleanup_volume_at_destination(self.pre_live_vol_data)
-        except Exception as e:
-            LOG.exception(e)
-            # Log the exception but no need to raise one because
-            # the VM is still on the source host.
+                 dict(volume=vol_drv.volume_id), instance=self.instance)
+        # Ensure the volume data is present before trying cleanup
+        if hasattr(self, 'pre_live_vol_data'):
+            try:
+                vol_drv.cleanup_volume_at_destination(self.pre_live_vol_data)
+            except Exception as e:
+                LOG.exception(e, instance=self.instance)
+                # Log the exception but no need to raise one because
+                # the VM is still on the source host.
 
 
 class LiveMigrationSrc(LiveMigration):
@@ -265,7 +268,7 @@ class LiveMigrationSrc(LiveMigration):
         lpar_w = vm.get_instance_wrapper(self.drvr.adapter, self.instance)
         self.lpar_w = lpar_w
 
-        LOG.debug('Dest Migration data: %s' % self.mig_data)
+        LOG.debug('Dest Migration data: %s', self.mig_data)
 
         # Check proc compatibility modes
         if (lpar_w.proc_compat_mode and lpar_w.proc_compat_mode not in
@@ -302,7 +305,8 @@ class LiveMigrationSrc(LiveMigration):
             vol_drv.pre_live_migration_on_source(vol_data)
         self.mig_data.vol_data = vol_data
 
-        LOG.debug('Src Migration data: %s' % self.mig_data)
+        LOG.debug('Src Migration data: %s', self.mig_data,
+                  instance=self.instance)
 
         # Create a FeedTask to scrub any orphaned mappings/storage associated
         # with this LPAR.  (Don't run it yet - we want to do the VOpt removal
@@ -328,7 +332,7 @@ class LiveMigrationSrc(LiveMigration):
         :param migrate_data: a PowerVMLiveMigrateData object
         """
         LOG.debug("Starting migration.", instance=self.instance)
-        LOG.debug("Migrate data: %s" % migrate_data)
+        LOG.debug("Migrate data: %s", migrate_data, instance=self.instance)
 
         # The passed in mig data has more info (dest data added), so replace
         self.mig_data = migrate_data
@@ -394,11 +398,11 @@ class LiveMigrationSrc(LiveMigration):
         # For each volume, make sure the source is cleaned
         for vol_drv in vol_drvs:
             LOG.info(_LI('Performing post migration for volume %(volume)s'),
-                     dict(volume=vol_drv.volume_id))
+                     dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
                 vol_drv.post_live_migration_at_source(migrate_data.vol_data)
             except Exception as e:
-                LOG.exception(e)
+                LOG.exception(e, instance=self.instance)
                 # Log the exception but no need to raise one because
                 # the VM is already moved.  By raising an exception that
                 # results in the VM being on the new host but the instance
@@ -461,9 +465,9 @@ class LiveMigrationSrc(LiveMigration):
             mig.migrate_abort(self.lpar_w)
         except Exception as ex:
             LOG.error(_LE("Abort of live migration has failed. "
-                          "This is non-blocking. "
-                          "Exception is logged below."))
-            LOG.exception(ex)
+                          "This is non-blocking. Exception is logged below."),
+                      instance=self.instance)
+            LOG.exception(ex, instance=self.instance)
 
     def migration_recover(self):
         """Recover migration if the migration failed for any reason. """
