@@ -20,13 +20,13 @@ from nova import exception
 from nova import utils
 
 from oslo_log import log as logging
-from taskflow import task
+from pypowervm.wrappers import network as pvm_net
 
 from nova_powervm import conf as cfg
-from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm.i18n import _LW
+from nova_powervm.virt.powervm.tasks import base as pvm_task
 from nova_powervm.virt.powervm import vif
 from nova_powervm.virt.powervm import vm
 
@@ -34,15 +34,7 @@ LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 
-class VirtualInterfaceUnplugException(exception.NovaException):
-
-    """Indicates that a VIF unplug failed."""
-    # TODO(thorst) symmetrical to the exception in base Nova.  Evaluate
-    # moving to Nova core.
-    msg_fmt = _("Virtual interface unplug failed")
-
-
-class UnplugVifs(task.Task):
+class UnplugVifs(pvm_task.PowerVMTask):
 
     """The task to unplug Virtual Network Interfaces from a VM."""
 
@@ -59,18 +51,14 @@ class UnplugVifs(task.Task):
                          slots used when a VIF is detached from the VM.
         """
         self.adapter = adapter
-        self.instance = instance
         self.network_infos = network_infos or []
         self.host_uuid = host_uuid
         self.slot_mgr = slot_mgr
 
-        super(UnplugVifs, self).__init__(name='unplug_vifs',
+        super(UnplugVifs, self).__init__(instance, 'unplug_vifs',
                                          requires=['lpar_wrap'])
 
-    def execute(self, lpar_wrap):
-        LOG.info(_LI('Unplugging the Network Interfaces to instance %s'),
-                 self.instance.name)
-
+    def execute_impl(self, lpar_wrap):
         # If the state is not in an OK state for deleting, then throw an
         # error up front.
         modifiable, reason = lpar_wrap.can_modify_io()
@@ -80,10 +68,10 @@ class UnplugVifs(task.Task):
                           'The reason reported by the system is: %(reason)s'),
                       {'inst': self.instance.name, 'reason': reason},
                       instance=self.instance)
-            raise VirtualInterfaceUnplugException()
+            raise exception.VirtualInterfaceUnplugException(reason=reason)
 
         # Get all the current Client Network Adapters (CNA) on the VM itself.
-        cna_w_list = vm.get_cnas(self.adapter, self.instance, self.host_uuid)
+        cna_w_list = vm.get_cnas(self.adapter, self.instance)
 
         # Walk through the VIFs and delete the corresponding CNA on the VM.
         for network_info in self.network_infos:
@@ -93,7 +81,7 @@ class UnplugVifs(task.Task):
         return cna_w_list
 
 
-class PlugVifs(task.Task):
+class PlugVifs(pvm_task.PowerVMTask):
 
     """The task to plug the Virtual Network Interfaces to a VM."""
 
@@ -101,7 +89,9 @@ class PlugVifs(task.Task):
                  slot_mgr):
         """Create the task.
 
-        Provides the 'vm_cnas' - the Virtual Machine's Client Network Adapters.
+        Provides 'vm_cnas' - the list of the Virtual Machine's Client Network
+        Adapters as they stand after all VIFs are plugged.  May be None, in
+        which case the Task requiring 'vm_cnas' should discover them afresh.
 
         :param virt_api: The VirtAPI for the operation.
         :param adapter: The pypowervm adapter.
@@ -114,37 +104,53 @@ class PlugVifs(task.Task):
         """
         self.virt_api = virt_api
         self.adapter = adapter
-        self.instance = instance
-        self.network_infos = network_infos
+        self.network_infos = network_infos or []
         self.host_uuid = host_uuid
         self.slot_mgr = slot_mgr
+        self.crt_network_infos, self.update_network_infos = [], []
+        self.cnas, self.vnics = None, None
 
-        super(PlugVifs, self).__init__(name='plug_vifs', provides='vm_cnas',
-                                       requires=['lpar_wrap'])
+        super(PlugVifs, self).__init__(
+            instance, 'plug_vifs', provides='vm_cnas', requires=['lpar_wrap'])
 
-    def execute(self, lpar_wrap):
-        LOG.info(_LI('Plugging the Network Interfaces to instance %s'),
-                 self.instance.name)
+    def _vif_exists(self, network_info):
+        """Does the instance have a CNA/VNIC (as appropriate) for a given net?
 
-        # Get the current adapters on the system
-        cna_w_list = vm.get_cnas(self.adapter, self.instance, self.host_uuid)
+        :param network_info: A network information dict.  This method expects
+                             it to contain keys 'vnic_type' (value is 'direct'
+                             for VNIC; otherwise assume CNA); and 'address'
+                             (MAC address).
+        :return: True if a CNA/VNIC (as appropriate) with the network_info's
+                 MAC address exists on the instance.  False otherwise.
+        """
+        # Are we looking for a VNIC or a CNA?
+        if network_info['vnic_type'] == 'direct':
+            if self.vnics is None:
+                self.vnics = vm.get_vnics(self.adapter, self.instance)
+            vifs = self.vnics
+        else:
+            if self.cnas is None:
+                self.cnas = vm.get_cnas(self.adapter, self.instance)
+            vifs = self.cnas
 
-        # Trim the VIFs down to the ones that haven't yet been created.
-        crt_network_infos = []
+        return network_info['address'] in [vm.norm_mac(v.mac) for v in vifs]
+
+    def execute_impl(self, lpar_wrap):
+        # We will have two types of network infos.  One is for newly created
+        # vifs.  The others are those that exist, but should be re-'treated'
         for network_info in self.network_infos:
-            for cna_w in cna_w_list:
-                if vm.norm_mac(cna_w.mac) == network_info['address']:
-                    break
+            if self._vif_exists(network_info):
+                self.update_network_infos.append(network_info)
             else:
-                crt_network_infos.append(network_info)
+                self.crt_network_infos.append(network_info)
 
-        # If there are no vifs to create, then just exit immediately.
-        if len(crt_network_infos) == 0:
+        # If there are no vifs to create or update, then just exit immediately.
+        if not self.crt_network_infos and not self.update_network_infos:
             return []
 
         # Check to see if the LPAR is OK to add VIFs to.
         modifiable, reason = lpar_wrap.can_modify_io()
-        if not modifiable:
+        if not modifiable and self.crt_network_infos:
             LOG.error(_LE('Unable to create VIF(s) for instance %(sys)s.  The '
                           'VM was in a state where VIF plugging is not '
                           'acceptable.  The reason from the system is: '
@@ -169,20 +175,34 @@ class PlugVifs(task.Task):
             self.instance.save()
             undo_host_change = True
 
+        # For existing VIFs that we just need to update, run the plug but do
+        # not wait for the neutron event as that likely won't be sent (it was
+        # already done).
+        for network_info in self.update_network_infos:
+            LOG.info(_LI("Updating VIF with mac %(mac)s for instance %(sys)s"),
+                     {'mac': network_info['address'],
+                      'sys': self.instance.name}, instance=self.instance)
+            vif.plug(self.adapter, self.host_uuid, self.instance,
+                     network_info, self.slot_mgr, new_vif=False)
+
         # For the VIFs, run the creates (and wait for the events back)
         try:
             with self.virt_api.wait_for_instance_event(
                     self.instance, self._get_vif_events(),
                     deadline=CONF.vif_plugging_timeout,
                     error_callback=self._vif_callback_failed):
-                for network_info in crt_network_infos:
+                for network_info in self.crt_network_infos:
                     LOG.info(_LI('Creating VIF with mac %(mac)s for instance '
                                  '%(sys)s'),
                              {'mac': network_info['address'],
                               'sys': self.instance.name},
                              instance=self.instance)
-                    vif.plug(self.adapter, self.host_uuid, self.instance,
-                             network_info, self.slot_mgr)
+                    new_vif = vif.plug(
+                        self.adapter, self.host_uuid, self.instance,
+                        network_info, self.slot_mgr, new_vif=True)
+                    if self.cnas is not None and isinstance(new_vif,
+                                                            pvm_net.CNA):
+                        self.cnas.append(new_vif)
         except eventlet.timeout.Timeout:
             LOG.error(_LE('Error waiting for VIF to be created for instance '
                           '%(sys)s'), {'sys': self.instance.name},
@@ -195,8 +215,7 @@ class PlugVifs(task.Task):
                 self.instance.host = old_host
                 self.instance.save()
 
-        # Return the list of created VIFs.
-        return cna_w_list
+        return self.cnas
 
     def _vif_callback_failed(self, event_name, instance):
         LOG.error(_LE('VIF Plug failure for callback on event '
@@ -219,12 +238,12 @@ class PlugVifs(task.Task):
         if (utils.is_neutron() and CONF.vif_plugging_is_fatal and
                 CONF.vif_plugging_timeout):
             return [('network-vif-plugged', network_info['id'])
-                    for network_info in self.network_infos
+                    for network_info in self.crt_network_infos
                     if not network_info.get('active', True)]
         else:
             return []
 
-    def revert(self, lpar_wrap, result, flow_failures):
+    def revert_impl(self, lpar_wrap, result, flow_failures):
         if not self.network_infos:
             return
 
@@ -235,8 +254,8 @@ class PlugVifs(task.Task):
                     instance=self.instance)
 
         # Get the current adapters on the system
-        cna_w_list = vm.get_cnas(self.adapter, self.instance, self.host_uuid)
-        for network_info in self.network_infos:
+        cna_w_list = vm.get_cnas(self.adapter, self.instance)
+        for network_info in self.crt_network_infos:
             try:
                 vif.unplug(self.adapter, self.host_uuid, self.instance,
                            network_info, self.slot_mgr, cna_w_list=cna_w_list)
@@ -247,14 +266,15 @@ class PlugVifs(task.Task):
                             instance=self.instance)
 
 
-class PlugMgmtVif(task.Task):
+class PlugMgmtVif(pvm_task.PowerVMTask):
 
     """The task to plug the Management VIF into a VM."""
 
     def __init__(self, adapter, instance, host_uuid, slot_mgr):
         """Create the task.
 
-        Requires the 'vm_cnas'.
+        Requires 'vm_cnas' from PlugVifs.  If None, this Task will retrieve the
+        VM's list of CNAs.
 
         Provides the mgmt_cna.  This may be none if no management device was
         created.  This is the CNA of the mgmt vif for the VM.
@@ -266,14 +286,14 @@ class PlugMgmtVif(task.Task):
                          slots used when a VIF is attached to the VM
         """
         self.adapter = adapter
-        self.instance = instance
         self.host_uuid = host_uuid
         self.slot_mgr = slot_mgr
 
         super(PlugMgmtVif, self).__init__(
-            name='plug_mgmt_vif', provides='mgmt_cna', requires=['vm_cnas'])
+            instance, 'plug_mgmt_vif', provides='mgmt_cna',
+            requires=['vm_cnas'])
 
-    def execute(self, vm_cnas):
+    def execute_impl(self, vm_cnas):
         # If configured to not use RMC mgmt vifs, then return None.  Need to
         # return None because the Config Drive step (which may be used...may
         # not be) required the mgmt vif.
@@ -287,20 +307,25 @@ class PlugMgmtVif(task.Task):
                      '%s'), self.instance.name, instance=self.instance)
         # Determine if we need to create the secure RMC VIF.  This should only
         # be needed if there is not a VIF on the secure RMC vSwitch
-        vswitch_w = vif.get_secure_rmc_vswitch(self.adapter, self.host_uuid)
-        if vswitch_w is None:
-            LOG.debug('No management VIF created for instance %s due to '
-                      'lack of Management Virtual Switch',
-                      self.instance.name)
+        vswitch = vif.get_secure_rmc_vswitch(self.adapter, self.host_uuid)
+        if vswitch is None:
+            LOG.warning(_LW('No management VIF created for instance %s due to '
+                        'lack of Management Virtual Switch'),
+                        self.instance.name)
             return None
 
         # This next check verifies that there are no existing NICs on the
         # vSwitch, so that the VM does not end up with multiple RMC VIFs.
-        for cna_w in vm_cnas:
-            if cna_w.vswitch_uri == vswitch_w.href:
-                LOG.debug('Management VIF already created for instance %s',
-                          self.instance.name)
-                return None
+        if vm_cnas is None:
+            has_mgmt_vif = vm.get_cnas(self.adapter, self.instance,
+                                       vswitch_uri=vswitch.href)
+        else:
+            has_mgmt_vif = vswitch.href in [cna.vswitch_uri for cna in vm_cnas]
+
+        if has_mgmt_vif:
+            LOG.debug('Management VIF already created for instance %s',
+                      self.instance.name)
+            return None
 
         # Return the created management CNA
         return vif.plug_secure_rmc_vif(
