@@ -14,10 +14,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
+
 from pypowervm.tasks import scsi_mapper as pvm_smap
 
 from oslo_log import log as logging
-from taskflow import task
 from taskflow.types import failure as task_fail
 
 from nova_powervm.virt.powervm.disk import driver as disk_driver
@@ -26,11 +27,12 @@ from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import media
 from nova_powervm.virt.powervm import mgmt
+from nova_powervm.virt.powervm.tasks import base as pvm_task
 
 LOG = logging.getLogger(__name__)
 
 
-class ConnectVolume(task.Task):
+class ConnectVolume(pvm_task.PowerVMTask):
 
     """The task to connect a volume to an instance."""
 
@@ -47,14 +49,14 @@ class ConnectVolume(task.Task):
         self.slot_mgr = slot_mgr
 
         super(ConnectVolume, self).__init__(
-            name='connect_vol_%s' % self.vol_id)
+            self.vol_drv.instance, 'connect_vol_%s' % self.vol_id)
 
-    def execute(self):
+    def execute_impl(self):
         LOG.info(_LI('Connecting volume %(vol)s to instance %(inst)s'),
                  {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
         self.vol_drv.connect_volume(self.slot_mgr)
 
-    def revert(self, result, flow_failures):
+    def revert_impl(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
         LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
@@ -79,7 +81,7 @@ class ConnectVolume(task.Task):
                          'error': e.message})
 
 
-class DisconnectVolume(task.Task):
+class DisconnectVolume(pvm_task.PowerVMTask):
 
     """The task to disconnect a volume from an instance."""
 
@@ -95,14 +97,14 @@ class DisconnectVolume(task.Task):
         self.vol_id = self.vol_drv.connection_info['data']['volume_id']
         self.slot_mgr = slot_mgr
         super(DisconnectVolume, self).__init__(
-            name='disconnect_vol_%s' % self.vol_id)
+            self.vol_drv.instance, 'disconnect_vol_%s' % self.vol_id)
 
-    def execute(self):
+    def execute_impl(self):
         LOG.info(_LI('Disconnecting volume %(vol)s from instance %(inst)s'),
                  {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
         self.vol_drv.disconnect_volume(self.slot_mgr)
 
-    def revert(self, result, flow_failures):
+    def revert_impl(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
         LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
@@ -128,7 +130,7 @@ class DisconnectVolume(task.Task):
                          'error': e.message})
 
 
-class CreateDiskForImg(task.Task):
+class CreateDiskForImg(pvm_task.PowerVMTask):
 
     """The Task to create the disk from an image in the storage."""
 
@@ -148,27 +150,20 @@ class CreateDiskForImg(task.Task):
                           than the image, the image size will be used.
         :param image_type: The image type. See disk/driver.py
         """
-        super(CreateDiskForImg, self).__init__(name='crt_disk_from_img',
-                                               provides='disk_dev_info')
+        super(CreateDiskForImg, self).__init__(
+            instance, 'crt_disk_from_img', provides='disk_dev_info')
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
         self.image_meta = image_meta
         self.disk_size = disk_size
         self.image_type = image_type
 
-    def execute(self):
-        LOG.info(_LI('Creating disk for instance: %s'), self.instance.name)
+    def execute_impl(self):
         return self.disk_dvr.create_disk_from_image(
             self.context, self.instance, self.image_meta, self.disk_size,
             image_type=self.image_type)
 
-    def revert(self, result, flow_failures):
-        # The parameters have to match the execute method, plus the response +
-        # failures even if only a subset are used.
-        LOG.warning(_LW('Image for instance %s to be deleted'),
-                    self.instance.name)
-
+    def revert_impl(self, result, flow_failures):
         # If there is no result, or its a direct failure, then there isn't
         # anything to delete.
         if result is None or isinstance(result, task_fail.Failure):
@@ -179,7 +174,7 @@ class CreateDiskForImg(task.Task):
         self.disk_dvr.delete_disks(self.context, self.instance, [result])
 
 
-class ConnectDisk(task.Task):
+class ConnectDisk(pvm_task.PowerVMTask):
 
     """The task to connect the disk to the instance."""
 
@@ -201,26 +196,22 @@ class ConnectDisk(task.Task):
                          the FeedTask is not provided, the updates will be run
                          immediately when the respective method is executed.
         """
-        super(ConnectDisk, self).__init__(name='connect_disk',
-                                          requires=['disk_dev_info'])
+        super(ConnectDisk, self).__init__(
+            instance, 'connect_disk', requires=['disk_dev_info'])
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
         self.stg_ftsk = stg_ftsk
 
-    def execute(self, disk_dev_info,):
-        LOG.info(_LI('Connecting disk to instance: %s'), self.instance.name)
+    def execute_impl(self, disk_dev_info,):
         self.disk_dvr.connect_disk(self.context, self.instance, disk_dev_info,
                                    stg_ftsk=self.stg_ftsk)
 
-    def revert(self, disk_dev_info, result, flow_failures):
-        LOG.warning(_LW('Disk image being disconnected from instance %s'),
-                    self.instance.name)
+    def revert_impl(self, disk_dev_info, result, flow_failures):
         # Note that the FeedTask is None - to force instant disconnect.
         self.disk_dvr.disconnect_image_disk(self.context, self.instance)
 
 
-class InstanceDiskToMgmt(task.Task):
+class InstanceDiskToMgmt(pvm_task.PowerVMTask):
 
     """Connect an instance's disk to the management partition, discover it.
 
@@ -244,19 +235,23 @@ class InstanceDiskToMgmt(task.Task):
         :param instance: The nova instance whose boot disk is to be connected.
         """
         super(InstanceDiskToMgmt, self).__init__(
-            name='connect_and_discover_instance_disk_to_mgmt',
+            instance, 'connect_and_discover_instance_disk_to_mgmt',
             provides=['stg_elem', 'vios_wrap', 'disk_path'])
         self.disk_dvr = disk_dvr
-        self.instance = instance
         self.stg_elem = None
         self.vios_wrap = None
         self.disk_path = None
 
-    def execute(self):
+    def execute_impl(self):
         """Map the instance's boot disk and discover it."""
-        LOG.info(_LI("Mapping boot disk of instance %(instance_name)s to "
-                     "management partition."),
-                 {'instance_name': self.instance.name})
+
+        # Search for boot disk on the Novalink partition
+        if self.disk_dvr.mp_uuid in self.disk_dvr.vios_uuids:
+            dev_name = self.disk_dvr.boot_disk_path_for_instance(
+                self.instance, self.disk_dvr.mp_uuid)
+            if dev_name is not None:
+                return None, None, dev_name
+
         self.stg_elem, self.vios_wrap = (
             self.disk_dvr.connect_instance_disk_to_mgmt(self.instance))
         new_maps = pvm_smap.find_maps(
@@ -277,7 +272,7 @@ class InstanceDiskToMgmt(task.Task):
         self.disk_path = mgmt.discover_vscsi_disk(the_map)
         return self.stg_elem, self.vios_wrap, self.disk_path
 
-    def revert(self, result, flow_failures):
+    def revert_impl(self, result, flow_failures):
         """Unmap the disk and then remove it from the management partition.
 
         We use this order to avoid rediscovering the device in case some other
@@ -303,7 +298,7 @@ class InstanceDiskToMgmt(task.Task):
         mgmt.remove_block_dev(self.disk_path)
 
 
-class RemoveInstanceDiskFromMgmt(task.Task):
+class RemoveInstanceDiskFromMgmt(pvm_task.PowerVMTask):
 
     """Unmap and remove an instance's boot disk from the mgmt partition."""
 
@@ -323,12 +318,11 @@ class RemoveInstanceDiskFromMgmt(task.Task):
         :param instance: The nova instance whose boot disk is to be connected.
         """
         self.disk_dvr = disk_dvr
-        self.instance = instance
         super(RemoveInstanceDiskFromMgmt, self).__init__(
-            name='remove_inst_disk_from_mgmt',
+            instance, 'remove_inst_disk_from_mgmt',
             requires=['stg_elem', 'vios_wrap', 'disk_path'])
 
-    def execute(self, stg_elem, vios_wrap, disk_path):
+    def execute_impl(self, stg_elem, vios_wrap, disk_path):
         """Unmap and remove an instance's boot disk from the mgmt partition.
 
         Input parameters ('requires') provided by InstanceDiskToMgmt task.
@@ -340,6 +334,9 @@ class RemoveInstanceDiskFromMgmt(task.Task):
         :param disk_path: The local path to the disk device to be removed, e.g.
                           '/dev/sde'
         """
+        # stg_elem is None if boot disk was not mapped to management partition
+        if stg_elem is None:
+            return
         LOG.info(_LI("Unmapping boot disk %(disk_name)s of instance "
                      "%(instance_name)s from management partition via Virtual "
                      "I/O Server %(vios_name)s."),
@@ -352,7 +349,7 @@ class RemoveInstanceDiskFromMgmt(task.Task):
         mgmt.remove_block_dev(disk_path)
 
 
-class CreateAndConnectCfgDrive(task.Task):
+class CreateAndConnectCfgDrive(pvm_task.PowerVMTask):
 
     """The task to create the configuration drive."""
 
@@ -379,26 +376,23 @@ class CreateAndConnectCfgDrive(task.Task):
                          immediately when the respective method is executed.
         """
         super(CreateAndConnectCfgDrive, self).__init__(
-            name='cfg_drive', requires=['lpar_wrap', 'mgmt_cna'])
+            instance, 'cfg_drive', requires=['lpar_wrap', 'mgmt_cna'])
         self.adapter = adapter
         self.host_uuid = host_uuid
-        self.instance = instance
         self.injected_files = injected_files
         self.network_info = network_info
         self.ad_pass = admin_pass
         self.mb = None
         self.stg_ftsk = stg_ftsk
 
-    def execute(self, lpar_wrap, mgmt_cna):
-        LOG.info(_LI('Creating Config Drive for instance: %s'),
-                 self.instance.name)
+    def execute_impl(self, lpar_wrap, mgmt_cna):
         self.mb = media.ConfigDrivePowerVM(self.adapter, self.host_uuid)
         self.mb.create_cfg_drv_vopt(self.instance, self.injected_files,
                                     self.network_info, lpar_wrap.uuid,
                                     admin_pass=self.ad_pass,
                                     mgmt_cna=mgmt_cna, stg_ftsk=self.stg_ftsk)
 
-    def revert(self, lpar_wrap, mgmt_cna, result, flow_failures):
+    def revert_impl(self, lpar_wrap, mgmt_cna, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
 
@@ -406,11 +400,16 @@ class CreateAndConnectCfgDrive(task.Task):
         if self.mb is None:
             return
 
-        # Delete the virtual optical media
-        self.mb.dlt_vopt(lpar_wrap.uuid)
+        # Delete the virtual optical media. If it fails we don't care.
+        try:
+            self.mb.dlt_vopt(lpar_wrap.uuid)
+        except Exception as e:
+            LOG.warning(_LW('Vopt removal as part of spawn reversion failed '
+                            'with: %(exc)s'), {'exc': six.text_type(e)},
+                        instance=self.instance)
 
 
-class DeleteVOpt(task.Task):
+class DeleteVOpt(pvm_task.PowerVMTask):
 
     """The task to delete the virtual optical."""
 
@@ -429,19 +428,18 @@ class DeleteVOpt(task.Task):
                          the FeedTask is not provided, the updates will be run
                          immediately when the respective method is executed.
         """
-        super(DeleteVOpt, self).__init__(name='vopt_delete')
+        super(DeleteVOpt, self).__init__(instance, 'vopt_delete')
         self.adapter = adapter
         self.host_uuid = host_uuid
-        self.instance = instance
         self.lpar_uuid = lpar_uuid
         self.stg_ftsk = stg_ftsk
 
-    def execute(self):
+    def execute_impl(self):
         media_builder = media.ConfigDrivePowerVM(self.adapter, self.host_uuid)
         media_builder.dlt_vopt(self.lpar_uuid, stg_ftsk=self.stg_ftsk)
 
 
-class DetachDisk(task.Task):
+class DetachDisk(pvm_task.PowerVMTask):
 
     """The task to detach the disk storage from the instance."""
 
@@ -463,23 +461,20 @@ class DetachDisk(task.Task):
                          immediately when the respective method is executed.
         :param disk_type: List of disk types to detach. None means detach all.
         """
-        super(DetachDisk, self).__init__(name='detach_storage',
-                                         provides='stor_adpt_mappings')
+        super(DetachDisk, self).__init__(
+            instance, 'detach_storage', provides='stor_adpt_mappings')
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
         self.stg_ftsk = stg_ftsk
         self.disk_type = disk_type
 
-    def execute(self):
-        LOG.info(_LI('Detaching disk storage adapters for instance %s'),
-                 self.instance.name)
+    def execute_impl(self):
         return self.disk_dvr.disconnect_image_disk(
             self.context, self.instance, stg_ftsk=self.stg_ftsk,
             disk_type=self.disk_type)
 
 
-class DeleteDisk(task.Task):
+class DeleteDisk(pvm_task.PowerVMTask):
 
     """The task to delete the backing storage."""
 
@@ -493,19 +488,16 @@ class DeleteDisk(task.Task):
         :param instance: The nova instance.
         """
         req = ['stor_adpt_mappings']
-        super(DeleteDisk, self).__init__(name='dlt_storage', requires=req)
+        super(DeleteDisk, self).__init__(instance, 'dlt_storage', requires=req)
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
 
-    def execute(self, stor_adpt_mappings):
-        LOG.info(_LI('Deleting storage disk for instance %s.'),
-                 self.instance.name)
+    def execute_impl(self, stor_adpt_mappings):
         self.disk_dvr.delete_disks(self.context, self.instance,
                                    stor_adpt_mappings)
 
 
-class SaveBDM(task.Task):
+class SaveBDM(pvm_task.PowerVMTask):
 
     """Task to save an updated block device mapping."""
 
@@ -516,17 +508,17 @@ class SaveBDM(task.Task):
         :param instance: The nova instance
         """
         self.bdm = bdm
-        self.instance = instance
-        super(SaveBDM, self).__init__(name='save_bdm_%s' % self.bdm.volume_id)
+        super(SaveBDM, self).__init__(
+            instance, 'save_bdm_%s' % self.bdm.volume_id)
 
-    def execute(self):
+    def execute_impl(self):
         LOG.info(_LI('Saving block device mapping for volume id %(vol_id)s '
                      'on instance %(inst)s.'),
                  {'vol_id': self.bdm.volume_id, 'inst': self.instance.name})
         self.bdm.save()
 
 
-class FindDisk(task.Task):
+class FindDisk(pvm_task.PowerVMTask):
 
     """The Task to find a disk and provide information to downstream tasks."""
 
@@ -541,15 +533,13 @@ class FindDisk(task.Task):
         :param instance: The nova instance.
         :param disk_type: One of the DiskType enum values.
         """
-        super(FindDisk, self).__init__(name='find_disk',
-                                       provides='disk_dev_info')
+        super(FindDisk, self).__init__(
+            instance, 'find_disk', provides='disk_dev_info')
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
         self.disk_type = disk_type
 
-    def execute(self):
-        LOG.info(_LI('Finding disk for instance: %s'), self.instance.name)
+    def execute_impl(self):
         disk = self.disk_dvr.get_disk_ref(self.instance, self.disk_type)
         if not disk:
             LOG.warning(_LW('Disk not found: %(disk_name)s'),
@@ -560,7 +550,7 @@ class FindDisk(task.Task):
         return disk
 
 
-class ExtendDisk(task.Task):
+class ExtendDisk(pvm_task.PowerVMTask):
 
     """Task to extend a disk."""
 
@@ -575,13 +565,12 @@ class ExtendDisk(task.Task):
         """
         self.disk_dvr = disk_dvr
         self.context = context
-        self.instance = instance
         self.disk_info = disk_info
         self.size = size
-        super(ExtendDisk, self).__init__(name='extend_disk_%s' %
-                                         disk_info['type'])
+        super(ExtendDisk, self).__init__(
+            instance, 'extend_disk_%s' % disk_info['type'])
 
-    def execute(self):
+    def execute_impl(self):
         LOG.info(_LI('Extending disk size of disk: %(disk)s size: %(size)s.'),
                  {'disk': self.disk_info['type'], 'size': self.size})
         self.disk_dvr.extend_disk(self.context, self.instance, self.disk_info,
