@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -17,7 +17,9 @@
 import abc
 import six
 
+from nova import context as ctx
 from nova import exception
+from nova import network as net_api
 from nova.network import linux_net
 from nova.network import model as network_model
 from nova import utils
@@ -42,7 +44,6 @@ from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import vm
 
-
 LOG = log.getLogger(__name__)
 
 SECURE_RMC_VSWITCH = 'MGMTSWITCH'
@@ -94,7 +95,7 @@ def _build_vif_driver(adapter, host_uuid, instance, vif):
         {'vif_type': vif['type'], 'instance': instance.name})
 
 
-def _push_vif_event(adapter, action, vif_w, vif_type):
+def _push_vif_event(adapter, action, vif_w, instance, vif_type):
     """Push a custom event to the REST server for a vif action (plug/unplug).
 
     This event prompts the neutron agent to mark the port up or down.
@@ -102,6 +103,7 @@ def _push_vif_event(adapter, action, vif_w, vif_type):
     :param adapter: The pypowervm adapter.
     :param action: The action taken on the vif - either 'plug' or 'unplug'
     :param vif_w: The pypowervm wrapper of the affected vif (CNA, VNIC, etc.)
+    :param instance: The nova instance for the event
     :param vif_type: The type of event source (pvm_sea, ovs, bridge,
                      pvm_sriov etc)
     """
@@ -111,9 +113,11 @@ def _push_vif_event(adapter, action, vif_w, vif_type):
     event = pvm_evt.Event.bld(adapter, data, detail)
     try:
         event = event.create()
-        LOG.info(_LI('Custom event push: %s'), str(event))
+        LOG.debug(_LI('Pushed custom event for consumption by neutron agent: '
+                      '%s'), str(event), instance=instance)
     except Exception:
-        LOG.error(_LE('Custom VIF event push failed.  %s'), str(event))
+        LOG.error(_LE('Custom VIF event push failed.  %s'), str(event),
+                  instance=instance)
         raise
 
 
@@ -144,8 +148,8 @@ def plug(adapter, host_uuid, instance, vif, slot_mgr, new_vif=True):
         vnet_w = vif_drv.plug(vif, slot_num, new_vif=new_vif)
     except pvm_ex.HttpError as he:
         # Log the message constructed by HttpError
-        LOG.exception(he.args[0])
-        raise exception.VirtualInterfacePlugException()
+        LOG.exception(he.args[0], instance=instance)
+        raise exception.VirtualInterfacePlugException(reason=he.args[0])
     # Other exceptions are (hopefully) custom VirtualInterfacePlugException
     # generated lower in the call stack.
 
@@ -156,7 +160,7 @@ def plug(adapter, host_uuid, instance, vif, slot_mgr, new_vif=True):
 
     # Push a custom event if we really plugged the vif
     if vnet_w is not None:
-        _push_vif_event(adapter, 'plug', vnet_w, vif['type'])
+        _push_vif_event(adapter, 'plug', vnet_w, instance, vif['type'])
 
     return vnet_w
 
@@ -179,10 +183,10 @@ def unplug(adapter, host_uuid, instance, vif, slot_mgr, cna_w_list=None):
         vnet_w = vif_drv.unplug(vif, cna_w_list=cna_w_list)
         # Push a custom event, but only if the vif existed in the first place
         if vnet_w:
-            _push_vif_event(adapter, 'unplug', vnet_w, vif['type'])
+            _push_vif_event(adapter, 'unplug', vnet_w, instance, vif['type'])
     except pvm_ex.HttpError as he:
         # Log the message constructed by HttpError
-        LOG.exception(he.args[0])
+        LOG.exception(he.args[0], instance=instance)
         raise exception.VirtualInterfaceUnplugException(reason=he.args[0])
 
     if vnet_w:
@@ -283,6 +287,14 @@ def plug_secure_rmc_vif(adapter, instance, host_uuid, slot_mgr):
     """
     # Gather the mac and slot number for the mgmt vif
     mac, slot_num = slot_mgr.build_map.get_mgmt_vea_slot()
+    if not mac:
+        # This is either a deploy case or rebuild case. For remote restart,
+        # mac will not be none, as it will be available from slot data.
+        # Deploy case - mac is None at both slot and instance_system_metadata
+        # and crt_cna will auto-generate it.
+        # Rebuild case - mac is none from slot data but is available
+        # at instance system_metadata.
+        mac = instance.system_metadata.get('mgmt_interface_mac')
 
     # Create the adapter.
     lpar_uuid = vm.get_pvm_uuid(instance)
@@ -291,8 +303,14 @@ def plug_secure_rmc_vif(adapter, instance, host_uuid, slot_mgr):
                             slot_num=slot_num, mac_addr=mac)
 
     # Save the mgmt vif to the slot map.
-    if not mac:
+    # For the rebuild case, mac will be present but not slot_num.
+    # For deploy case, both will be none. We want to register cna in both cases
+    if not slot_num:
         slot_mgr.register_cna(cna_w)
+    if cna_w.mac != mac:
+        # Update instance system metadata to store instance management
+        # interface mac address.
+        instance.system_metadata.update({'mgmt_interface_mac': cna_w.mac})
 
     return cna_w
 
@@ -353,22 +371,27 @@ class PvmVifDriver(object):
             LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
                             'instance %(inst)s.  The VIF was not found on '
                             'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name})
+                        {'mac': vif['address'], 'inst': self.instance.name},
+                        instance=self.instance)
             return None
 
         LOG.info(_LI('Deleting VIF with mac %(mac)s for instance %(inst)s.'),
-                 {'mac': vif['address'], 'inst': self.instance.name})
+                 {'mac': vif['address'], 'inst': self.instance.name},
+                 instance=self.instance)
         try:
             cna_w.delete()
         except Exception as e:
-            LOG.exception(e)
+            LOG.error(_LE('Unable to unplug VIF with mac %(mac)s for instance '
+                          '%(inst)s.'),
+                      {'mac': vif['address'], 'inst': self.instance.name},
+                      instance=self.instance)
+            LOG.exception(e, instance=self.instance)
             raise exception.VirtualInterfaceUnplugException(
-                _LE('Unable to unplug VIF with mac %(mac)s for instance '
-                    '%(inst)s.') % {'mac': vif['address'],
-                                    'inst': self.instance.name})
+                reason=six.text_type(e))
         return cna_w
 
-    def _find_cna_for_vif(self, cna_w_list, vif):
+    @staticmethod
+    def _find_cna_for_vif(cna_w_list, vif):
         """Finds the PowerVM CNA for a given Nova VIF.
 
         :param cna_w_list: The list of Client Network Adapter wrappers from
@@ -461,7 +484,7 @@ class PvmSeaVifDriver(PvmVifDriver):
         if not vlan:
             vlan = int(vif['details']['vlan'])
 
-        LOG.debug("Creating SEA based VIF with VLAN %s" % str(vlan))
+        LOG.debug("Creating SEA based VIF with VLAN %s", str(vlan))
         cna_w = pvm_cna.crt_cna(self.adapter, self.host_uuid, lpar_uuid, vlan,
                                 mac_addr=vif['address'], slot_num=slot_num)
 
@@ -472,7 +495,8 @@ class PvmSeaVifDriver(PvmVifDriver):
 class PvmLioVifDriver(PvmVifDriver):
     """An abstract VIF driver that uses Linux I/O to host."""
 
-    def get_trunk_dev_name(self, vif):
+    @staticmethod
+    def get_trunk_dev_name(vif):
         """Returns the device name for the trunk adapter.
 
         A given VIF in the Linux I/O model will have a trunk adapter and a
@@ -578,7 +602,8 @@ class PvmLBVifDriver(PvmLioVifDriver):
             LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
                             'instance %(inst)s.  The VIF was not found on '
                             'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name})
+                        {'mac': vif['address'], 'inst': self.instance.name},
+                        instance=self.instance)
             return None
 
         # Find and delete the trunk adapters
@@ -594,7 +619,7 @@ class PvmLBVifDriver(PvmLioVifDriver):
                             '%(bridge)s. Error: %(error)s'),
                         {'dev_name': dev_name,
                          'bridge': vif['network']['bridge'],
-                         'error': e.message})
+                         'error': e.message}, instance=self.instance)
         for trunk in trunks:
             trunk.delete()
 
@@ -610,17 +635,37 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
             return None
 
         physnet = vif.get_physical_network()
+        if not physnet:
+            # Get physnet from neutron network if not present in vif
+            # TODO(svenkat): This section of code will be eliminated in
+            # pike release. Design will be in place to fix any vif
+            # that has physical_network missing. The fix will be in
+            # compute startup code.
+            net_id = vif['network']['id']
+            admin_context = ctx.get_admin_context()
+            napi = net_api.API()
+            network = napi.get(admin_context, net_id)
+            physnet = network.physical_network
+
         LOG.debug("Plugging vNIC SR-IOV vif for physical network "
                   "'%(physnet)s' into instance %(inst)s.",
-                  {'physnet': physnet, 'inst': self.instance.name})
+                  {'physnet': physnet, 'inst': self.instance.name},
+                  instance=self.instance)
+
+        # Get the msys
+        msys = pvm_ms.System.get(self.adapter)[0]
+        # Physical ports for the given port label
+        pports_w = sriovtask.find_pports_for_portlabel(physnet, self.adapter,
+                                                       msys)
+        pports = [pport.loc_code for pport in pports_w]
 
-        # Physical ports for the given physical network
-        pports = vif['details']['physical_ports']
         if not pports:
             raise exception.VirtualInterfacePlugException(
-                _("Unable to find SR-IOV physical ports for physical "
-                  "network '%(physnet)s' (instance %(inst)s).  VIF: %(vif)s") %
-                {'physnet': physnet, 'inst': self.instance.name, 'vif': vif})
+                _("Unable to find acceptable Ethernet ports on physical "
+                  "network '%(physnet)s' for instance %(inst)s for SRIOV "
+                  "based VIF with MAC address %(vif_mac)s.") %
+                {'physnet': physnet, 'inst': self.instance.name,
+                 'vif_mac': vif['address']})
 
         # MAC
         mac_address = pvm_util.sanitize_mac_for_api(vif['address'])
@@ -639,7 +684,8 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
             allowed_vlans=pvm_util.VLANList.NONE,
             allowed_macs=pvm_util.MACList.NONE)
 
-        sriovtask.set_vnic_back_devs(vnic, pports, redundancy=redundancy,
+        sriovtask.set_vnic_back_devs(vnic, pports, sys_w=msys,
+                                     redundancy=redundancy,
                                      capacity=capacity, check_port_status=True)
 
         return vnic.create(parent_type=pvm_lpar.LPAR,
@@ -653,7 +699,8 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
             LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
                             'instance %(inst)s.  No matching vNIC was found '
                             'on the instance.  VIF: %(vif)s'),
-                        {'mac': mac, 'inst': self.instance.name, 'vif': vif})
+                        {'mac': mac, 'inst': self.instance.name, 'vif': vif},
+                        instance=self.instance)
             return None
         vnic.delete()
         return vnic
@@ -678,15 +725,11 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         :return: The new vif that was created.  Only returned if new_vif is
                  set to True.  Otherwise None is expected.
         """
-        # Only call the parent if it is truly a new VIF
-        if new_vif:
-            cna_w = super(PvmOvsVifDriver, self).plug(vif, slot_num)
-        else:
-            cna_w = None
+        cna_w = super(PvmOvsVifDriver, self).plug(vif, slot_num,
+                                                  new_vif=new_vif)
         dev_name = self.get_trunk_dev_name(vif)
         # There will only be one trunk wrap, as we have created with just the
-        # mgmt lpar.  Next step is to set the device up and connect to the OVS
-        utils.execute('ip', 'link', 'set', dev_name, 'up', run_as_root=True)
+        # mgmt lpar.  Next step is to connect to the OVS.
         mtu = vif['network'].get_meta('mtu')
         linux_net.create_ovs_vif_port(vif['network']['bridge'], dev_name,
                                       self.get_ovs_interfaceid(vif),
@@ -695,7 +738,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
 
         return cna_w
 
-    def get_ovs_interfaceid(self, vif):
+    @staticmethod
+    def get_ovs_interfaceid(vif):
         """Returns the interface id to set for a given VIF.
 
         When a VIF is plugged for an Open vSwitch, it needs to have the
@@ -728,7 +772,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
             LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
                             'instance %(inst)s.  The VIF was not found on '
                             'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name})
+                        {'mac': vif['address'], 'inst': self.instance.name},
+                        instance=self.instance)
             return None
 
         # Find and delete the trunk adapters
