@@ -238,7 +238,7 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # manager.  Given that, we need to update the order of the WWPNs.
         # The first WWPN is the one that is logged into the fabric and this
         # will now indicate that our WWPN is logged in.
-        LOG.debug('Post live migrate volume store: %s' % mig_vol_stor,
+        LOG.debug('Post live migrate volume store: %s', mig_vol_stor,
                   instance=self.instance)
         for fabric in self._fabric_names():
             # We check the mig_vol_stor to see if this fabric has already been
@@ -254,8 +254,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                 # Flip the WPWNs
                 c_wwpns = port_map[1].split()
                 c_wwpns.reverse()
-                LOG.debug('Flipping WWPNs, ports: %s wwpns: %s' %
-                          (port_map, c_wwpns), instance=self.instance)
+                LOG.debug('Flipping WWPNs, ports: %s wwpns: %s',
+                          port_map, c_wwpns, instance=self.instance)
                 # Get the new physical WWPN.
                 vfc_map = pvm_vfcm.find_vios_for_vfc_wwpns(vios_wraps,
                                                            c_wwpns)[1]
@@ -305,7 +305,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         LOG.info(_LI("Instance %(inst)s has not yet defined a WWPN on "
                      "fabric %(fabric)s.  Appropriate WWPNs will be "
                      "generated."),
-                 {'inst': self.instance.name, 'fabric': fabric})
+                 {'inst': self.instance.name, 'fabric': fabric},
+                 instance=self.instance)
         return True
 
     def _hosts_wwpn(self, port_maps):
@@ -401,9 +402,10 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # were already configured.
         for fabric in self._fabric_names():
             fc_state = self._get_fabric_state(fabric)
-            LOG.info(_LI("NPIV wwpns fabric state=%(st)s for "
-                         "instance %(inst)s") %
-                     {'st': fc_state, 'inst': self.instance.name})
+            LOG.info(_LI(
+                "NPIV wwpns fabric state=%(st)s for instance %(inst)s"),
+                {'st': fc_state, 'inst': self.instance.name},
+                instance=self.instance)
 
             if self._is_initial_wwpn(fc_state, fabric):
                 # Get a set of WWPNs that are globally unique from the system.
@@ -473,7 +475,7 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             return npiv_port_maps
 
         # If ANY of the VIOS ports were not there, rebuild the port maps
-        LOG.debug("Rebuild existing_npiv_port_maps=%s. Reset fabric state." %
+        LOG.debug("Rebuild existing_npiv_port_maps=%s. Reset fabric state.",
                   npiv_port_maps)
         v_wwpns = []
         for port_map in npiv_port_maps:
@@ -483,7 +485,7 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # Derive new maps and don't preserve existing maps
         npiv_port_maps = pvm_vfcm.derive_npiv_map(
             vios_wraps, self._fabric_ports(fabric), v_wwpns, preserve=False)
-        LOG.debug("Rebuilt port maps: %s" % npiv_port_maps)
+        LOG.debug("Rebuilt port maps: %s", npiv_port_maps)
         self._set_fabric_meta(fabric, npiv_port_maps)
         LOG.warning(_LW("Had to update the system metadata for the WWPNs "
                         "due to incorrect physical WWPNs on fabric "
@@ -513,7 +515,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             vios_w = pvm_vfcm.find_vios_for_port_map(vios_wraps, npiv_port_map)
             if vios_w is None:
                 LOG.error(_LE("Mappings were not able to find a proper VIOS. "
-                              "The port mappings were %s."), npiv_port_maps)
+                              "The port mappings were %s."), npiv_port_maps,
+                          instance=self.instance)
                 raise exc.VolumeAttachFailed(
                     volume_id=volume_id, instance_name=self.instance.name,
                     reason=_("Unable to find a Virtual I/O Server that "
@@ -586,15 +589,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             else:
                 LOG.warning(_LW("No storage connections found between the "
                                 "Virtual I/O Servers and FC Fabric "
-                                "%(fabric)s."), {'fabric': fabric})
-
-    def host_name(self):
-        """Derives the host name that should be used for the storage device.
-
-        :return: The host name.
-        """
-        host = CONF.host if len(CONF.host) < 20 else CONF.host[:20]
-        return host + '_' + self.instance.name
+                                "%(fabric)s."), {'fabric': fabric},
+                            instance=self.instance)
 
     def _set_fabric_state(self, fabric, state):
         """Sets the fabric state into the instance's system metadata.
@@ -607,8 +603,9 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
          FS_INST_MAPPED: Fabric is mapped with the nova instance.
         """
         meta_key = self._sys_fabric_state_key(fabric)
-        LOG.info(_LI("Setting Fabric state=%(st)s for instance=%(inst)s") %
-                 {'st': state, 'inst': self.instance.name})
+        LOG.info(_LI("Setting Fabric state=%(st)s for instance=%(inst)s"),
+                 {'st': state, 'inst': self.instance.name},
+                 instance=self.instance)
         self.instance.system_metadata[meta_key] = state
 
     def _get_fabric_state(self, fabric):
@@ -661,7 +658,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         LOG.info(_LI("Fabric %(fabric)s wwpn metadata will be set to "
                      "%(meta)s for instance %(inst)s"),
                  {'fabric': fabric, 'meta': ",".join(meta_elems),
-                  'inst': self.instance.name})
+                  'inst': self.instance.name},
+                 instance=self.instance)
 
         # Clear out the original metadata.  We may be reducing the number of
         # keys (ex. reschedule) so we need to just delete what we had before
@@ -706,7 +704,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                             "fabric %(fabric)s.  May not have connected to "
                             "the fabric yet or fabric configuration was "
                             "recently modified."),
-                        {'inst': self.instance.name, 'fabric': fabric})
+                        {'inst': self.instance.name, 'fabric': fabric},
+                        instance=self.instance)
             return []
 
         wwpns = self.instance.system_metadata[meta_key]
